@@ -1,33 +1,37 @@
-package Email::Valid;
 require 5.006;
 use strict;
 use warnings;
-use vars qw( $VERSION $RFC822PAT %AUTOLOAD $AUTOLOAD $NSLOOKUP_PAT
-             @NSLOOKUP_PATHS $Details $Resolver $Nslookup_Path
-             $DNS_Method $TLD $Debug );
+package Email::Valid;
+{
+  $Email::Valid::VERSION = '1.192';
+}
+# ABSTRACT: Check validity of Internet email addresses
+our (
+  $RFC822PAT,
+  $Details, $Resolver, $Nslookup_Path,
+  $Debug,
+);
 
 use Carp;
 use IO::File;
 use Mail::Address;
 use File::Spec;
+use Scalar::Util 'blessed';
 
-use bytes;
-
-$VERSION = '0.185';
-
-%AUTOLOAD = (
+our %AUTOLOAD = (
   fqdn     => 1,
   fudge    => 1,
   mxcheck  => 1,
   tldcheck => 1,
   local_rules => 1,
+  localpart => 1,
 );
 
-$NSLOOKUP_PAT = 'preference|serial|expire|mail\s+exchanger';
-@NSLOOKUP_PATHS = File::Spec->path();
+our $NSLOOKUP_PAT = 'preference|serial|expire|mail\s+exchanger';
+our @NSLOOKUP_PATHS = File::Spec->path();
 
 # initialize if already loaded, better in prefork mod_perl environment
-$DNS_Method = defined $Net::DNS::VERSION ? 'Net::DNS' : '';
+our $DNS_Method = defined $Net::DNS::VERSION ? 'Net::DNS' : '';
 unless ($DNS_Method) {
     __PACKAGE__->_select_dns_method;
 }
@@ -56,7 +60,9 @@ sub _initialize {
   $self->{tldcheck}    = 0;
   $self->{fudge}       = 0;
   $self->{fqdn}        = 1;
+  $self->{allow_ip}    = 1;
   $self->{local_rules} = 0;
+  $self->{localpart}   = 1;
   $self->{details}     = $Details = undef;
 }
 
@@ -102,7 +108,7 @@ sub rfc822 {
   my %args = $self->_rearrange([qw( address )], \@_);
 
   my $addr = $args{address} or return $self->details('rfc822');
-  $addr = $addr->address if eval { $addr->isa('Mail::Address') };
+  $addr = $addr->address if (blessed($addr) && $addr->isa('Mail::Address'));
 
   return $self->details('rfc822')
     if $addr =~ /\P{ASCII}/ or $addr !~ m/^$RFC822PAT$/o;
@@ -139,10 +145,17 @@ sub _net_dns_query {
 
   $Resolver = Net::DNS::Resolver->new unless defined $Resolver;
 
-  my $packet = $Resolver->send($host, 'A') or croak $Resolver->errorstring;
-  return 1 if $packet->header->ancount;
+  my $packet = $Resolver->send($host, 'MX') or croak $Resolver->errorstring;
+  if ($packet->header->ancount) {
+    my $mx = ($packet->answer)[0]->exchange;
+    if ($mx eq '.' or $mx eq '') {
+      return $self->details('mx'); # Null MX
+    } else {
+      return 1;
+    }
+  }
 
-  $packet = $Resolver->send($host, 'MX') or croak $Resolver->errorstring;
+  $packet = $Resolver->send($host, 'A') or croak $Resolver->errorstring;
   return 1 if $packet->header->ancount;
 
   return $self->details('mx');
@@ -234,7 +247,7 @@ sub _host {
   my $self = shift;
   my $addr = shift;
 
-  $addr = $addr->address if eval { $addr->isa('Mail::Address') };
+  $addr = $addr->address if (blessed($addr) && $addr->isa('Mail::Address'));
 
   my $host = ($addr =~ /^.*@(.*)$/ ? $1 : $addr);
   $host =~ s/\s+//g;
@@ -279,6 +292,14 @@ sub _local_rules {
   1;
 }
 
+sub _valid_local_part {
+  my ($self, $localpart) = @_;
+
+  return 0 unless defined $localpart and length $localpart <= 64;
+
+  return 1;
+}
+
 sub _valid_domain_parts {
   my ($self, $string) = @_;
 
@@ -312,7 +333,7 @@ sub address {
                                     local_rules )], \@_);
 
   my $addr = $args{address} or return $self->details('rfc822');
-  $addr = $addr->address if eval { $addr->isa('Mail::Address') };
+  $addr = $addr->address if (blessed($addr) && $addr->isa('Mail::Address'));
 
   $addr = $self->_fudge( $addr ) if $args{fudge};
   $self->rfc822( -address => $addr ) or return undef;
@@ -321,18 +342,32 @@ sub address {
 
   $addr or return $self->details('rfc822'); # This should never happen
 
+  if (length($addr->address) > 254) {
+    return $self->details('address_too_long');
+  }
+
   if ($args{local_rules}) {
     $self->_local_rules( $addr->user, $addr->host )
       or return $self->details('local_rules');
   }
 
-  if ($args{fqdn}) {
-    no warnings 'uninitialized'; # valid domain parts might return undef
-    $self->_valid_domain_parts($addr->host) > 1
-      or return $self->details('fqdn');
+  if ($args{localpart}) {
+    $self->_valid_local_part($addr->user) > 0
+      or return $self->details('localpart');
   }
 
-  if ($args{tldcheck}) {
+  my $ip_ok = $args{allow_ip} && $addr->host =~ /\A\[
+    (?:[0-9]{1,3}\.){3}[0-9]{1,3}
+  /x;
+
+  if (! $ip_ok && $args{fqdn}) {
+    my $domain_parts = $self->_valid_domain_parts($addr->host);
+
+    return $self->details('fqdn')
+      unless $ip_ok || ($domain_parts && $domain_parts > 1);
+  }
+
+  if (! $ip_ok && $args{tldcheck}) {
     $self->tld( $addr->host ) or return $self->details('tldcheck');
   }
 
@@ -348,7 +383,7 @@ sub address {
 sub AUTOLOAD {
   my $self = shift;
   my $type = ref($self) || die "$self is not an object";
-  my $name = $AUTOLOAD;
+  my $name = our $AUTOLOAD;
 
   $name =~ s/.*://;
   return if $name eq 'DESTROY';
@@ -464,14 +499,21 @@ $RFC822PAT =~ s/\n//g;
 
 __END__
 
+=pod
+
 =head1 NAME
 
 Email::Valid - Check validity of Internet email addresses
 
+=head1 VERSION
+
+version 1.192
+
 =head1 SYNOPSIS
 
   use Email::Valid;
-  print (Email::Valid->address('maurice@hevanet.com') ? 'yes' : 'no');
+  my $address = Email::Valid->address('maurice@hevanet.com');
+  print ($address ? 'yes' : 'no');
 
 =head1 DESCRIPTION
 
@@ -510,6 +552,7 @@ individual methods below of details.
  -tldcheck
  -fudge
  -fqdn
+ -allow_ip
  -local_rules
 
 =item mx ( <ADDRESS>|<DOMAIN> )
@@ -526,11 +569,6 @@ the behavior of the resolver (e.g. change the default tcp_timeout
 value) by manipulating the global Net::DNS::Resolver instance stored in
 $Email::Valid::Resolver.
 
-=item tld ( <ADDRESS> )
-
-This method determines whether the domain part of an address is in a recognized
-top-level domain.
-
 =item rfc822 ( <ADDRESS> )
 
 This method determines whether an address conforms to the RFC822
@@ -544,11 +582,34 @@ common addressing errors.  Currently, this results in the removal of
 spaces in AOL addresses, and the conversion of commas to periods in
 Compuserve addresses.  The default is false.
 
+=item allow_ip ( <TRUE>|<FALSE> )
+
+Specifies whether a "domain literal" is acceptable as the domain part.  That
+means addresses like:  C<rjbs@[1.2.3.4]>
+
+The checking for the domain literal is stricter than the RFC and looser than
+checking for a valid IP address, I<but this is subject to change>.
+
+The default is true.
+
 =item fqdn ( <TRUE>|<FALSE> )
 
 Species whether addresses passed to address() must contain a fully
 qualified domain name (FQDN).  The default is true.
 
+B<Please note!>  FQDN checks only occur for non-domain-literals.  In other
+words, if you have set C<allow_ip> and the address ends in a bracketed IP
+address, the FQDN check will not occur.
+
+=item tld ( <ADDRESS> )
+
+This method determines whether the domain part of an address is in a
+recognized top-level domain.
+
+B<Please note!>  TLD checks only occur for non-domain-literals.  In other
+words, if you have set C<allow_ip> and the address ends in a bracketed IP
+address, the TLD check will not occur.
+
 =item local_rules ( <TRUE>|<FALSE> )
 
 Specifies whether addresses passed to address() should be tested
@@ -581,6 +642,7 @@ If the last call to address() returned undef, you can call this
 method to determine why it failed.  Possible values are:
 
  rfc822
+ localpart
  local_rules
  fqdn
  mxcheck
@@ -630,20 +692,6 @@ wrapping the call in an eval block:
   };
   warn "an error was encountered: $@" if $@;
 
-=head1 BUGS
-
-Email::Valid should work with Perl for Win32.  In my experience,
-however, Net::DNS queries seem to take an extremely long time when
-a record cannot be found.
-
-=head1 AUTHOR
-
-Copyright 1998-2003, Maurice Aubrey E<lt>maurice@hevanet.comE<gt>.
-All rights reserved.
-
-This module is free software; you may redistribute it and/or
-modify it under the same terms as Perl itself.
-
 =head1 CREDITS
 
 Significant portions of this module are based on the ckaddr program
@@ -666,4 +714,15 @@ bug fixes:
 
 Mail::Address, Net::DNS, Net::Domain::TLD, perlfaq9
 
+=head1 AUTHOR
+
+Maurice Aubrey <maurice@hevanet.com>
+
+=head1 COPYRIGHT AND LICENSE
+
+This software is copyright (c) 1998 by Maurice Aubrey.
+
+This is free software; you can redistribute it and/or modify it under
+the same terms as the Perl 5 programming language system itself.
+
 =cut
